#include "ws.h"
#include "pc.h"
#include <libwebsockets.h>

void UserData::send(uint8_t *buf, size_t size) {
	if (!send_flag) {
		if (send_buf == NULL) {
			send_buf = new uint8_t[LWS_SEND_BUFFER_PRE_PADDING + BUFSIZE + LWS_SEND_BUFFER_POST_PADDING];
		}
		send_size = LWS_SEND_BUFFER_PRE_PADDING + size + LWS_SEND_BUFFER_POST_PADDING;
		memset(send_buf, 0, send_size);
		memcpy(send_buf + LWS_SEND_BUFFER_PRE_PADDING, buf, size);
		send_flag = true;
	}
}

static int callback_http(libwebsocket_context *context, 
	libwebsocket *wsi, libwebsocket_callback_reasons reason, 
	void *user, void *in, size_t len) {
	return 0;
}

static int callback_pc8001(libwebsocket_context *context, 
	libwebsocket *wsi, libwebsocket_callback_reasons reason, 
	void *user, void *in, size_t len) {
	UserData *u = (UserData *)user;
	switch (reason) {
	case LWS_CALLBACK_ESTABLISHED:
		memset(user, 0, sizeof(UserData));
		break;
	case LWS_CALLBACK_CLOSED:
		pthread_cancel(u->pc_thread);
		pthread_join(u->pc_thread, NULL);
		break;
	case LWS_CALLBACK_PROTOCOL_DESTROY:
		delete[] u->send_buf;
		delete u->filename;
		break;
	case LWS_CALLBACK_SERVER_WRITEABLE:
		if (u->send_flag) {
			libwebsocket_write(wsi, u->send_buf, u->send_size, LWS_WRITE_BINARY);
			u->send_flag = false;
		}
		libwebsocket_callback_on_writable(context, wsi);
		break;
	case LWS_CALLBACK_RECEIVE:
		if (!u->pc_thread) {
			u->filename = new std::string(len > 3 ? (char *)in + 3 : "");
			pthread_create(&u->pc_thread, NULL, exec_pc, user);
			libwebsocket_callback_on_writable(context, wsi);
		}
		else if (len == 2) {
			processKeyData(*u->pc, ((uint8_t *)in)[0], ((uint8_t *)in)[1]);
		}
		break;
	default:
		break;
	}
	return 0;
}

int main() {
	static libwebsocket_protocols protocols[] = {
		{ "http", callback_http, 0 },
		{ "pc8001-protocol", callback_pc8001, sizeof(UserData) },
		{ NULL, NULL, 0 }
	};
	lws_context_creation_info info;
	memset(&info, 0, sizeof info);
	info.port = 8080;
	info.protocols = protocols;
	info.gid = info.uid = -1;
	libwebsocket_context *context = libwebsocket_create_context(&info);
	if (context == NULL) {
		fprintf(stderr, "libwebsocket init failed.\n");
		return 1;
	}
	while (1) {
		libwebsocket_service(context, 0);
		usleep(1000);
	}
	libwebsocket_context_destroy(context);
	return 0;
}

